% Text Mining with MATLAB 2nd Edition
% www.http://www.textmininglab.net/

% CHAPTER 6: The Structure of Language

clear
disp('Example 6.1 ------------------------------------------------------');
s1 = "I gave Peter my 2 theater tickets. ";                         %(6.1a)
s2 = "He lives in San Francisco :) ";
strdoc = s1+s2;
tokdoc = tokenizedDocument(strdoc);                                 %(6.1b)
whos strdoc tokdoc                                                  %(6.1c)
pause;

disp('Example 6.2 ------------------------------------------------------');
tokdoc.Vocabulary                                                    %(6.2)
pause;

disp('Example 6.3 ------------------------------------------------------');
tokenDetails(tokdoc)                                                 %(6.3)
pause;

disp('Example 6.4 ------------------------------------------------------');
see_inflections = ["see","sees","saw","seen","seeing"];             %(6.4a)
normalizeWords(see_inflections,'Style','lemma') % Reduces to lemmas  (6.4b)
normalizeWords(see_inflections,'Style','stem') % Reduces to stems    (6.4b)
pause; 

disp('Example 6.5 ------------------------------------------------------');
walk_inflections = ["walk","walks","walked","walking"];             %(6.5a)
normalizeWords(walk_inflections,'Style','lemma') % Reduces to lemmas (6.5b)
normalizeWords(walk_inflections,'Style','stem') % Reduces to stems   (6.5b)
pause;

disp('Example 6.6 ------------------------------------------------------');
tokdoc = addLemmaDetails(tokdoc); % Adds lemmas                      (6.6a)
tokdoc = addPartOfSpeechDetails(tokdoc); % Adds POS                  (6.6b)
doctbl = tokenDetails(tokdoc);                                      %(6.6c)
doctbl(:,{'Token','Lemma','PartOfSpeech'})
pause;

disp('Example 6.7 ------------------------------------------------------');
doc = "Tokenization isn't static";                                  %(6.7a)
otk = tokenizedDocument(doc) % Original tokenization                %(6.7b)
rtk = addPartOfSpeechDetails(otk) % Retokenized document            %(6.7c)
pause;

disp('Example 6.8 ------------------------------------------------------');
stk = addPartOfSpeechDetails(otk,'RetokenizeMethod','none')         %(6.8a)
stktbl = tokenDetails(stk); stktbl.PartOfSpeech'                    %(6.8b)
pause;

disp('Example 6.9 ------------------------------------------------------');
tokdoc = addSentenceDetails(tokdoc); % Adds sentence number         %(6.9a)
doctbl = tokenDetails(tokdoc);                                      %(6.9b)  
doctbl(:,{'Token','SentenceNumber','Lemma','PartOfSpeech'})
pause;

disp('Example 6.10 -----------------------------------------------------');
nerdoc = addEntityDetails(tokdoc); % Adds named entity labels      %(6.10a)
doctbl = tokenDetails(nerdoc);                                     %(6.10b)
doctbl(:,{'Token','SentenceNumber','PartOfSpeech','Entity'})
pause;

disp('Example 6.11 -----------------------------------------------------');
% Identifies named entities without changing tokenization          %(6.11a)
keeptok = addEntityDetails(tokdoc,'RetokenizeMethod','none');
doctbl = tokenDetails(keeptok);                                    %(6.10b)
idx = doctbl.Entity ~= 'non-entity'; % gets indexes for named entities
doctbl.Token(idx)' % displays the tokens of the identified named entities
doctbl.Entity(idx)' % displays the labels of the identified named entities
pause;
